# Usage: LEVEL=INFO N=10000 bundle exec ruby hacking/benchmarks.rb

require 'log4r'
require 'gallus'
require 'stringio'
require 'benchmark'

LEVEL = ENV.fetch('LEVEL', 'INFO')

$gallus = Gallus::Log.configure('test') do |log|
  log.level = LEVEL
  log.output << Gallus::Output::Stream.new(StringIO.new, Gallus::Format::SimpleLog.new)
end

N = ENV.fetch('N', 10000).to_i
calls = []

N.times do
  %w[debug info warn error].each do |level|
    calls << level
  end
end

Log4r::IOOutputter.new('stream', StringIO.new, formatter: Log4r::PatternFormatter.new(pattern: "%l @ %d $ %C - %m"))

$log4r = Log4r::Logger.new('test')
$log4r.level = Log4r.const_get(LEVEL)
$log4r.add('stream')

payload = { one: 1, two: "Two", three: Object.new, four: "Yada! yada! yada!" }

Benchmark.bm do |x|
  x.report("Log4r: ") do
    calls.dup.shuffle.each do |level|
      $log4r.send(level) do
        formatted_payload = payload.map { |k,v| "#{k}=#{v.inspect}" }.join(" ")
        "This is fancy little message; #{formatted_payload}"
      end
    end
  end

  x.report("Gallus: ") do
    calls.dup.shuffle.each do |level|
      $gallus.send(level, "This is fancy little message", payload)
    end
  end
end
