require 'active_support/core_ext/hash/keys'
require 'perfume/super_object'
require 'perfume/logging'

module Perfume
  # Public: The pattern is simple, our Service is a struct with init arguments that
  # additionally is callable. It also comes with class level `call` method that
  # executes stuff on new instance of the object.
  class Service < SuperObject
    include Logging

    # Public: Shorthand to instantionante with arguments and perform call in one shot.
    # You should actually always intend to use this method as it's easier to mock
    # in testing.
    def self.call(args = {}, &block)
      new(args).call(&block)
    end

    # Public: Implement this one on your own. Your logic goes here.
    def call(&block)
      raise NotImplementedError
    end

    protected

    def system_call(*args, &block)
      Shell::SystemCall.call(*args, &block)
    end

    def exec(*args, &block)
      Shell::Exec.call(*args, &block)
    end
  end

  # Public: Shorthand to define a Service with accessors.
  def self.Service(*names, &block)
    Class.new(Service, &block).tap { |klass| klass.args_accessor(*names) }
  end
end
