require 'perfume/service'

module Perfume
  module Shell
    # Internal: Executing shell commands isn't good practice. Especially it's bad when
    # calling ruby methods like `exec` or `system`. This is dangerous and completely
    # untestable. So here it comes base wrapper around shell calls.
    #
    # Check Perfume::Shell::Exec and Perfume::Shell:SystemCall for two most common
    # use cases.
    class Base < Perfume::Service(:cmd, :root, :log)
      def init
        @root = Pathname.new(@root)
      end

      def log
        @log or self.class.log
      end

      def defaults
        { root: Dir.pwd }
      end

      def before
      end

      def to_s
        cmd.to_s
      end

      protected

      def fail!(msg = "Shell command execution failed!")
        raise ExecutionError, msg
      end
    end
  end
end
