require 'pathname'
require 'forwardable'

module Perfume
  module Testing
    # Public: Sometimes you have to test file contents or some biggest chunks of data. Put them in fixtures
    # folder and load using this helper class. Example:
    #
    #   MY_FIXTURE_FILES = FixtureFiles.load('path/to/fixtures/*.txt')
    #   MY_FIXTURE_FILES.each { |content| ... }
    #   puts MY_FIXTURE_FILES['filename.txt']
    #
    class FixtureFiles
      extend Forwardable
      def_delegators :@fixtures, :[], :size, :each

      include Enumerable

      class << self
        alias :load :new
      end

      def initialize(dir)
        @fixtures = load_fixtures(dir)
      end

      private

      def load_fixtures(dir)
        {}.tap do |fixtures|
          Pathname.glob(dir.to_s) do |entry|
            fixtures[entry.basename.to_s] = entry.read if entry.file?
          end
        end
      end
    end
  end
end
